@@ -1,5 +1,5 @@
 /* 
- * bdajax v1.3
+ * bdajax v1.4
  * 
  * Requires:
  * - jQuery 1.6.4
@@ -33,6 +33,9 @@
                 }
             }
         });
+        // XXX: probably ajax forms get a separate ``ajax:form`` directive
+        //      in markup.
+        bdajax.bind_ajax_form(context);
         for (var binder in bdajax.binders) {
             bdajax.binders[binder](context);
         }
@@ -45,7 +48,7 @@
         // That we assume at '/login'.
         default_403: '/login',
         
-        // object for 3rd party binders
+        // object for hooking up JS binding functions after ajax calls
         binders: {},
         
         // ajax spinner handling
@@ -55,29 +58,29 @@
             _request_count: 0,
             
             elem: function() {
-                if (bdajax.spinner._elem == null) {
-                    bdajax.spinner._elem = $('#ajax-spinner');
+                if (this._elem == null) {
+                    this._elem = $('#ajax-spinner');
                 }
-                return bdajax.spinner._elem;
+                return this._elem;
             },
             
             show: function() {
-                bdajax.spinner._request_count++;
-                if (bdajax.spinner._request_count > 1) {
+                this._request_count++;
+                if (this._request_count > 1) {
                     return;
                 }
-                bdajax.spinner.elem().show();
+                this.elem().show();
             },
             
             hide: function(force) {
-                bdajax.spinner._request_count--;
+                this._request_count--;
                 if (force) {
-                    bdajax.spinner._request_count = 0;
-                    bdajax.spinner.elem().hide();
+                    this._request_count = 0;
+                    this.elem().hide();
                     return;
-                } else if (bdajax.spinner._request_count <= 0) {
-                    bdajax.spinner._request_count = 0;
-                    bdajax.spinner.elem().hide();
+                } else if (this._request_count <= 0) {
+                    this._request_count = 0;
+                    this.elem().hide();
                 }
             }
         },
@@ -107,8 +110,8 @@
         },
         
         parsetarget: function(target) {
-            var url = bdajax.parseurl(target);
-            var params = bdajax.parsequery(target);
+            var url = this.parseurl(target);
+            var params = this.parsequery(target);
             if (!params) { params = {}; }
             return {
                 url: url,
@@ -119,8 +122,8 @@
         request: function(options) {
             if (options.url.indexOf('?') != -1) {
                 var addparams = options.params;
-                options.params = bdajax.parsequery(options.url);
-                options.url = bdajax.parseurl(options.url);
+                options.params = this.parsequery(options.url);
+                options.url = this.parseurl(options.url);
                 for (var key in addparams) {
                     options.params[key] = addparams[key];
                 }
@@ -150,7 +153,7 @@
                               request.statusText || error);
                 bdajax.spinner.hide(true);
             }
-            bdajax.spinner.show();
+            this.spinner.show();
             $.ajax({
                 url: options.url,
                 dataType: options.type,
@@ -162,8 +165,8 @@
         },
         
         action: function(options) {
-            options.success = bdajax._ajax_action_success;
-            bdajax._perform_ajax_action(options);
+            options.success = this._ajax_action_success;
+            this._perform_ajax_action(options);
         },
         
         fiddle: function(payload, selector, mode) {
@@ -185,13 +188,13 @@
             if (!definitions) {
                 return;
             }
-            bdajax.spinner.hide();
+            this.spinner.hide();
             var definition, target;
             for (var idx in definitions) {
                 definition = definitions[idx];
                 if (definition.type == 'action') {
-                    target = bdajax.parsetarget(definition.target);
-                    bdajax.action({
+                    target = this.parsetarget(definition.target);
+                    this.action({
                         url: target.url,
                         params: target.params,
                         name: definition.name,
@@ -199,9 +202,26 @@
                         selector: definition.selector
                     });
                 } else if (definition.type == 'event') {
-                    bdajax.trigger(definition.name,
-                                   definition.selector,
-                                   definition.target);
+                    this.trigger(definition.name,
+                                 definition.selector,
+                                 definition.target);
+                } else if (definition.type == 'overlay') {
+                    if (definition.close) {
+                        var elem = $(definition.selector);
+                        var overlay = elem.data('overlay');
+                        if (overlay) {
+                            overlay.close();
+                        }
+                    } else {
+                        var target = this.parsetarget(definition.target);
+                        this.overlay({
+                            action: definition.action,
+                            selector: definition.selector,
+                            content_selector: definition.content_selector,
+                            url: target.url,
+                            params: target.params
+                        });
+                    }
                 } else if (definition.type == 'message') {
                     if (definition.flavor) {
                         var flavors = ['message', 'info', 'warning', 'error'];
@@ -210,16 +230,16 @@
                         }
                         switch (definition.flavor) {
                             case 'message':
-                                bdajax.message(definition.payload);
+                                this.message(definition.payload);
                                 break;
                             case 'info':
-                                bdajax.info(definition.payload);
+                                this.info(definition.payload);
                                 break;
                             case 'warning':
-                                bdajax.warning(definition.payload);
+                                this.warning(definition.payload);
                                 break;
                             case 'error':
-                                bdajax.error(definition.payload);
+                                this.error(definition.payload);
                                 break;
                         }
                     } else {
@@ -234,25 +254,33 @@
         
         trigger: function(name, selector, target) {
             var evt = $.Event(name);
-            evt.ajaxtarget = bdajax.parsetarget(target);
+            evt.ajaxtarget = this.parsetarget(target);
             $(selector).trigger(evt);
         },
         
         overlay: function(options) {
-            var elem = $('#ajax-overlay');
+            var selector = '#ajax-overlay';
+            if (options.selector) {
+                selector = options.selector;
+            }
+            var content_selector = '.overlay_content';
+            if (options.content_selector) {
+                content_selector = options.content_selector;
+            }
+            var elem = $(selector);
             elem.removeData('overlay');
             var url, params;
             if (options.target) {
-                var target = bdajax.parsetarget(options.target);
+                var target = this.parsetarget(options.target);
                 url = target.url;
                 params = target.params;
             } else {
                 url = options.url;
                 params = options.params;
             }
-            bdajax._perform_ajax_action({
+            this._perform_ajax_action({
                 name: options.action,
-                selector: '#ajax-overlay-content',
+                selector: selector + ' ' + content_selector,
                 mode: 'inner',
                 url: url,
                 params: params,
@@ -268,8 +296,9 @@
                             loadSpeed: 200
                         },
                         onClose: function() {
-                            var overlay = this.getOverlay();
-                            $('#ajax-overlay-content', overlay).html('');
+                            var content = $(content_selector,
+                                            this.getOverlay());
+                            content.html('');
                         },
                         oneInstance: false,
                         closeOnClick: true,
@@ -311,19 +340,19 @@
         error: function(message) {
             $("#ajax-message .message").removeClass('error warning info')
                                        .addClass('error');
-            bdajax.message(message);
+            this.message(message);
         },
         
         info: function(message) {
             $("#ajax-message .message").removeClass('error warning info')
                                        .addClass('info');
-            bdajax.message(message);
+            this.message(message);
         },
         
         warning: function(message) {
             $("#ajax-message .message").removeClass('error warning info')
                                        .addClass('warning');
-            bdajax.message(message);
+            this.message(message);
         },
         
         dialog: function(options, callback) {
@@ -355,6 +384,25 @@
             elem.data('overlay').load();
         },
         
+        // bind ajax form handling to all forms providing ajax css class
+        bind_ajax_form: function(context) {
+            var ajaxform = $('form.ajax', context);
+            ajaxform.append('<input type="hidden" name="ajax" value="1" />');
+            ajaxform.attr('target', 'ajaxformresponse');
+            ajaxform.unbind().bind('submit', function(event) {
+                bdajax.spinner.show();
+            });
+        },
+        
+        // called by iframe response, renders form (i.e. if validation errors)
+        render_ajax_form: function(payload, selector, mode) {
+            if (!payload) {
+                return;
+            }
+            this.spinner.hide();
+            this.fiddle(payload, selector, mode);
+        },
+        
         _dispatching_handler: function(event) {
             event.preventDefault();
             event.stopPropagation();
@@ -387,11 +435,11 @@
         
         _handle_ajax_event: function(elem) {
             var target = elem.attr('ajax:target');
-            var defs = bdajax._defs_to_array(elem.attr('ajax:event'));
+            var defs = this._defs_to_array(elem.attr('ajax:event'));
             for (var i = 0; i < defs.length; i++) {
                 var def = defs[i];
                 def = def.split(':');
-                bdajax.trigger(def[0], def[1], target);
+                this.trigger(def[0], def[1], target);
             }
         },
         
@@ -409,7 +457,7 @@
             options.params['bdajax.action'] = options.name;
             options.params['bdajax.mode'] = options.mode;
             options.params['bdajax.selector'] = options.selector;
-            bdajax.request({
+            this.request({
                 url: bdajax.parseurl(options.url) + '/ajaxaction',
                 type: 'json',
                 params: options.params,
@@ -422,12 +470,12 @@
             if (event.ajaxtarget) {
                 target = event.ajaxtarget;
             } else {
-                target = bdajax.parsetarget(elem.attr('ajax:target'));
+                target = this.parsetarget(elem.attr('ajax:target'));
             }
-            var actions = bdajax._defs_to_array(elem.attr('ajax:action'));
+            var actions = this._defs_to_array(elem.attr('ajax:action'));
             for (var i = 0; i < actions.length; i++) {
                 var defs = actions[i].split(':');
-                bdajax.action({
+                this.action({
                     name: defs[0],
                     selector: defs[1],
                     mode: defs[2],
@@ -442,14 +490,28 @@
             if (event.ajaxtarget) {
                 target = event.ajaxtarget;
             } else {
-                target = bdajax.parsetarget(elem.attr('ajax:target'));
+                target = this.parsetarget(elem.attr('ajax:target'));
+            }
+            var overlay_attr = elem.attr('ajax:overlay');
+            if (overlay_attr.indexOf(':') > -1) {
+                var defs = overlay_attr.split(':');
+                var options = {
+                    action: defs[0],
+                    selector: defs[1],
+                    url: target.url,
+                    params: target.params
+                };
+                if (defs.length == 3) {
+                    options.content_selector = defs[2];
+                }
+                this.overlay(options);
+            } else {
+                this.overlay({
+                    action: overlay_attr,
+                    url: target.url,
+                    params: target.params
+                });
             }
-            var actionname = elem.attr('ajax:overlay');
-            bdajax.overlay({
-                action: actionname,
-                url: target.url,
-                params: target.params
-            });
         },
         
         _defs_to_array: function(str) {
